#ifdef WIN32
#pragma comment(lib, "c:\\TopfieldDisk\\DataFiles\\SDK\\TAP_SDK_DLL")
#include "TAP_Emulations.h"
#endif

#include "tap.h"
#include "RasterFontBrowser.h"
#include "Config.h"
#include "Utils.h"

#define OPTIONS_FILENAME "FontManager.ini"
#define DATA_BUFFER_SIZE_ini 1024

static char  buffer[256];
static char  *dataBuffer_ini;
static int   dataBufferPtr_ini;

void WriteIniFile()
{
	TYPE_File	*writeFile;

	if ( TAP_Hdd_Exist( OPTIONS_FILENAME ) ) TAP_Hdd_Delete( OPTIONS_FILENAME );	// Just delete any old copies

	TAP_Hdd_Create( OPTIONS_FILENAME, ATTR_PROGRAM );						// Create the file

	writeFile = TAP_Hdd_Fopen( OPTIONS_FILENAME );
	if ( writeFile == NULL ) return; 										// Check we can open it

	TAP_Hdd_Fwrite( dataBuffer_ini, DATA_BUFFER_SIZE_ini, 1, writeFile );	// dump the whole buffer in one hit

	TAP_Hdd_Fclose( writeFile );
}

void WriteStrToIniBuf( char *str )											// add str to current end of buffer
{																			// and move end out
	int count, i;

    count = (int) strlen(str);
	
	for ( i=0; i<count; i++)
	{
		dataBuffer_ini[dataBufferPtr_ini+i] = str[i];							// copy accross 1 char at a time
	}
	dataBufferPtr_ini += i;
}

void SaveConfigurationToFile( void )
{
	ChangeDirSettings();

	dataBuffer_ini = TAP_MemAlloc( DATA_BUFFER_SIZE_ini );				// Buffer the write data to memory before writing all in one hit
	memset( dataBuffer_ini, '\0', DATA_BUFFER_SIZE_ini );				// set the whole buffer to the string termination character (null)
	dataBufferPtr_ini = 0;

	TAP_SPrint(buffer, "%d\r\n", version_number);
	WriteStrToIniBuf( buffer );

	TAP_SPrint(buffer, "%s\r\n", font_name);
	WriteStrToIniBuf( buffer );

	TAP_SPrint(buffer, "%d\r\n", antialias_override);
	WriteStrToIniBuf( buffer );

	TAP_SPrint(buffer, "%d\r\n", hook_all);
	WriteStrToIniBuf( buffer );

	WriteIniFile();										// write all the data in one pass
	TAP_MemFree(dataBuffer_ini);										// must return the memory back to the heap
}


//------------------------------------------------------------------
//
char *ReadIniField( void )
{
	int i;
	char readChar;
	char *oldPtr;

	oldPtr = (char *) dataBuffer_ini + dataBufferPtr_ini;

	i=0;
	while ( dataBufferPtr_ini < DATA_BUFFER_SIZE_ini )					// Bounds check
	{
		readChar = dataBuffer_ini[dataBufferPtr_ini];					// read the file one character at a time

		if ( readChar == '\t' ) break;									// looking for TAB delimiter,
		if ( readChar == '\n' ) break;									// or end of line.
		if ( readChar == '\r' )
		{																// Carriage return is followed by new line -> discard them both.
			dataBuffer_ini[dataBufferPtr_ini] = '\0';
			dataBufferPtr_ini++;
			break;
		}
		if ( readChar == '\0' ) break;									// Null terminator signifies end of file

		dataBufferPtr_ini++;
	}

	dataBuffer_ini[dataBufferPtr_ini] = '\0';							// add a terminator to the string
	dataBufferPtr_ini++;
	return oldPtr;														// return a pointer to the start of the current field
}

dword ReadIniDecimal( void )
{
	dword	tmp;
	int i;
	
	strcpy( buffer, ReadIniField() );									
	tmp = 0;
	i = 0;

	while ( buffer[i] != '\0' )
	{
	    tmp = (tmp * 10) + (buffer[i] - '0');
		i++;
	}

	return tmp;
}


void SetConfigurationVariables( void )
{
	version_number				= ReadIniDecimal();

	strcpy(font_name,			ReadIniField());
	antialias_override			= ReadIniDecimal();
	hook_all					= ReadIniDecimal();
}

bool ReadConfigurationFile(void)
{
	TYPE_File *readFile;
	dword fileLength;

	ChangeDirSettings();

	if ( ! TAP_Hdd_Exist( OPTIONS_FILENAME ) ) return FALSE;			// check the photo file exits in the current directory
	
	readFile = TAP_Hdd_Fopen( OPTIONS_FILENAME );
	if ( readFile == NULL ) return FALSE;								// and we can open it ok

	dataBuffer_ini = TAP_MemAlloc( DATA_BUFFER_SIZE_ini );				// Read the data in one hit (quicker), then process from RAM
	memset( dataBuffer_ini, '\0', DATA_BUFFER_SIZE_ini );					// set the whole buffer to the string termination character (null)
	dataBufferPtr_ini = 0;

	fileLength = TAP_Hdd_Flen( readFile );								// how big is the file
	if ( fileLength > DATA_BUFFER_SIZE_ini  ) fileLength = DATA_BUFFER_SIZE_ini;	// ensure we don't overflow the buffer

	TAP_Hdd_Fread( dataBuffer_ini, fileLength, 1, readFile );			
	TAP_Hdd_Fclose( readFile );

	SetConfigurationVariables();

	TAP_MemFree( dataBuffer_ini );										// must return the memory back to the heap

	return TRUE;
}

void LoadConfiguration( void )
{
	if ((!ReadConfigurationFile()))  // If file is invalid, recreate a new one.
	{
		strcpy(font_name,		"Default");
		antialias_override		= FALSE;
		hook_all				= FALSE;
		version_number			= 1;

		SaveConfigurationToFile();
	}

	//hook_pointers.hook_all = hook_all;
}

void switchHookAll()
{
	hook_all = hook_all ? FALSE : TRUE;
	//hook_pointers.hook_all = hook_all;
}