#ifndef RASTERFONTBROWSER_H
#define RASTERFONTBROWSER_H

#define GETR(rgb)       ((byte)((rgb)>>10)&0x1F)
#define GETG(rgb)       ((byte)(((word)(rgb))>>5)&0x1F)
#define GETB(rgb)       ((byte)(rgb&0x1F))

#define PUTPIXEL_OR(IMAGE, X, Y, COLOR)     IMAGE[Y*width+X]|= COLOR
#define PUTPIXEL_SET(IMAGE, X, Y, COLOR)    IMAGE[Y*width+X] = COLOR
#define GETPIXEL(IMAGE, X, Y, COLOR)        COLOR = IMAGE[Y*width+X]

#define NO_ALLOC
#define FONT_OVERRIDE  100

typedef struct
{
    long  slot_advance_x;
    long  slot_advance_y;
    long  slot_bitmap_left;
    long  slot_bitmap_top;
    long  bitmap_rows;
    long  bitmap_width;
    long  offset;
} TableItem;

typedef struct
{
    char *m_data;

    word           m_foreColor;
    word           m_backColor;
    word           m_antialiasmap[16];
    TableItem     *m_bmpHeaderArray;
    long           m_fontHeight;
    long           m_fontBaseline;
    int            m_bShifted;

#ifdef NO_ALLOC
    unsigned char *m_grayBuffer;
    word          *m_wordBuffer;
#endif
	word		  antialias_factor;
} FONT;
//static dword antialias_factor = 10;

typedef struct
{
	char font_name[100];
	bool small_exists;
	bool medium_exists;
	bool large_exists;
} _font;

_font fonts[512];
int current_font, font_count;
FONT small_font;
FONT medium_font;
FONT large_font;

#define JUMP_CMD 0x08000000
#define NOP_CMD 0x00000000
#define REL_ADDR(x) (((dword)(x) & 0x0fffffff) >> 2)

/*typedef struct
{
	dword hook_all;
    dword hooked_PutS[2];
	dword hooked_PutString[2];
	dword hooked_PutStringAf[2];
	dword hooked_GetW[2];
} HOOK_POINTERS;

HOOK_POINTERS hook_pointers;*/

typedef struct {
	void*	(*MemAlloc)(dword size);
	int	    (*MemFree)(const void *addr);
	int	(*Osd_FillBox)(word rgn, dword x, dword y, dword w, dword h, dword color);
	int	(*Osd_DrawPixmap)(word rgn, dword x, dword y, dword w, dword h, void *pixmap, bool sprite, byte dataFormat );
	int	(*Osd_GetPixel)(word rgn, dword x, dword y, void *pixel);
	int (*Osd_GetBaseInfo)( TYPE_OsdBaseInfo *info );
	int	(*Osd_PutString)(word rgn, dword x, dword y, dword maxX, const char * str, word fcolor, word bcolor, byte fntType, byte fntSize, byte nextLine);
	int	(*Osd_PutStringAf)(word rgn, dword x, dword y, dword maxX, const char * str, word fcolor, word bcolor, byte fntType, byte fntSize, byte nextLine);
	int	(*Osd_PutS)(word rgn, dword x, dword y, dword maxX, const char * str, word fcolor, word bcolor, byte fntType, byte fntSize, byte bDot, byte align);
	int	(*Osd_GetW)( const char *str, byte fntType, byte fntSize );
	void	(*Print)(const void *fmt, ...);
	void	(*Delay)(dword dm10);
} TAP_Funcs;
extern TAP_Funcs _TF;

#define LOAD_CHAR(font, pos, bitmap)    *(bitmap) = (font)->m_data + (font)->m_bmpHeaderArray[(pos)].offset;

void loadFont();
bool loadFonts();
bool loadNewFonts();
void switchFont();
void deleteFont();
void deleteFontFiles();
bool isValidFont(int i);
void shuffleFontsDown(int i);
void adjustAntialiasing();
int Load_Font(FONT *font, char *fileName);
void Delete_Font(FONT *font);
int TAP_Osd_PutS_Font(word rgn, dword x, dword y, dword maxX, unsigned char *str, word fcolor, word bcolor, FONT *font, byte align, byte bDot);
int TAP_Osd_PutS_FontEx(word rgn, dword x, dword y, dword maxX, dword maxY, int baseline_shift, unsigned char *str, word fcolor, word bcolor, FONT *font, byte align);
int TAP_Osd_PutS_FontL(word rgn, dword x, dword y, dword maxX, unsigned char *str, FONT *font, byte align, byte bDot);
int Draw_Font_String(word rgn, dword x, dword y, char *str, char* fontname,  word foreColor, word backColor);

//////////////////////////////////////////////////////////////////////////
int CalcSize(FONT *font, unsigned char* text, int* width, int *text_length, byte bDot, int *len);
void SetColors(FONT *font, word foreColor, word antialias_backColor, word backColor, word bytes_per_pixel);

//////////////////////////////////////////////////////////////////////////
// Hook interface
#define HOOK

#ifdef HOOK

int TAP_Osd_GetW_HOOK(const char *str, byte fntType, byte fntSize);
int TAP_Osd_PutS_HOOK(word rgn, dword x, dword y, dword maxX, const char *str, word fcolor, word bcolor, byte fntType, byte fntSize, byte bDot, byte align);
int TAP_Osd_PutString_HOOK(word rgn, dword x, dword y, dword maxX, const char * str, word fcolor, word bcolor, byte fntType, byte fntSize, byte nextLine);
int TAP_Osd_PutStringAf_HOOK(word rgn, dword x, dword y, dword maxX, const char * str, word fcolor, word bcolor, byte fntType, byte fntSize, byte nextLine);
//void applyHook();
//void removeHook();

#endif // HOOK

#endif // RASTERFONTBROWSER_H